<?php
/**
 * RButtonColumn class file.
 *
 * @author Rupert
 * @copyright Rupert, 2012
 * @license http://www.opensource.org/licenses/mit-license.php MIT License
 */

Yii::import('zii.widgets.grid.CButtonColumn');

/**
 * RButtonColumn represents a grid view column that renders one or several buttons.
 *
 * By default, it will display four buttons, "view", "update" and "delete", which triggers the corresponding
 * actions on the model of the row, and "create" in the column header to create a new model.
 *
 * By configuring {@link buttons} and {@link template} properties, the column can display other buttons
 * and customize the display order of the buttons (likewise the {@link headerButtons} and {@link headerTemplate}
 * properties for the column header).
 *
 * @author Rupert
 */

class RButtonColumn extends CButtonColumn {
	/**
	 * The constant indicating a button of link (anchor) type
	 * @var constant string
	 */
	const LINK_BUTTON = 'RLinkButton';
	/**
	 * The constant indicating a button of submit (form) type
	 * @var constant string
	 */
	const SUBMIT_BUTTON = 'RSubmitButton';
	
	/**
	 * @var array the HTML options for the filter cell tag.
	 */
	public $filterHtmlOptions=array();
	/**
	 * @var string the template that is used to render the content in the header cell.
	 * These default tokens are recognized: {create}. If the {@link headerButtons} property
	 * defines additional buttons, their IDs are also recognized here. For example, if a button named 'selectAll'
	 * is declared in {@link headerButtons}, we can use the token '{selectAll}' here to specify where to display the button.
	 */
	public $headerTemplate = '{create}';
	/**
	 * @var string the label for the create button. Defaults to "Create".
	 * Note that the label will not be HTML-encoded when rendering.
	 */
	public $createButtonLabel;
	/**
	 * @var string the image URL for the create button. If not set, an integrated image will be used.
	 * You may set this property to be false to render a text link instead.
	 */
	public $createButtonImageUrl;
	/**
	 * @var string a PHP expression that is evaluated and whose result is used
	 * as the URL for the create button. In this expression, the variable
	 * <code>$this</code> is the column object.
	 */
	public $createButtonUrl='Yii::app()->controller->createUrl("create")';
	/**
	 * @var array the HTML options for the view button tag.
	 */
	public $createButtonOptions=array('class'=>'create');
	/**
	 * @var array the configuration for additional header buttons. Each array element specifies a single button
	 * which has the following format:
	 * <pre>
	 * 'buttonID' => array(
	 *     'label'=>'...',     // text label of the button
	 *     'url'=>'...',       // a PHP expression for generating the URL of the button
	 *     'imageUrl'=>'...',  // image URL of the button. If not set or false, a text link is used
	 *     'options'=>array(...), // HTML options for the button tag
	 *     'click'=>'...',     // a JS function to be invoked when the button is clicked
	 *     'visible'=>'...',   // a PHP expression for determining whether the button is visible
	 *     'type'=>'...',      // the type of button to be rendered (self::LINK_BUTTON or self::SUBMIT_BUTTON)
	 * )
	 * </pre>
	 *
	 * Note that in order to display these additional buttons, the {@link headerTemplate} property needs to
	 * be configured so that the corresponding button IDs appear as tokens in the template.
	 */
	public $headerButtons = array();
	/**
	 * @var array the configuration for additional buttons. Each array element specifies a single button
	 * which has the following format:
	 * <pre>
	 * 'buttonID' => array(
	 *     'label'=>'...',     // text label of the button
	 *     'url'=>'...',       // a PHP expression for generating the URL of the button
	 *     'imageUrl'=>'...',  // image URL of the button. If not set or false, a text link is used
	 *     'options'=>array(...), // HTML options for the button tag
	 *     'click'=>'...',     // a JS function to be invoked when the button is clicked
	 *     'visible'=>'...',   // a PHP expression for determining whether the button is visible
	 *     'type'=>'...',      // the type of button to be rendered (self::LINK_BUTTON or self::SUBMIT_BUTTON)
	 * )
	 * </pre>
	 * In the PHP expression for the 'url' option and/or 'visible' option, the variable <code>$row</code>
	 * refers to the current row number (zero-based), and <code>$data</code> refers to the data model for
	 * the row.
	 *
	 * Note that in order to display these additional buttons, the {@link template} property needs to
	 * be configured so that the corresponding button IDs appear as tokens in the template.
	 */
	public $buttons=array();
	
	/**
	 * Initializes the column.
	 * This method registers necessary client script for the button column.
	 */
	public function init()
	{
		if(!isset($this->filterHtmlOptions['class']))
			$this->filterHtmlOptions['class']=$this->id.'-filter';
		
		$this->initDefaultButtons();

		foreach($this->headerButtons as $id=>$button)
		{
			if(strpos($this->headerTemplate,'{'.$id.'}')===false)
				unset($this->headerButtons[$id]);
			else if(isset($button['click']) || isset($button['submit']))
			{
				if(!isset($button['options']['id']))
					$this->headerButtons[$id]['options']['id']=$id;
				if(isset($button['click']) && strpos($button['click'],'js:')!==0)
					$this->headerButtons[$id]['click']='js:'.$button['click'];
				if(isset($button['submit']) && isset($button['type']) && $button['type'] == self::SUBMIT_BUTTON && strpos($button['submit'],'js:')!==0)
					$this->headerButtons[$id]['submit']='js:'.$button['submit'];
			}
		}
		foreach($this->buttons as $id=>$button)
		{
			if(strpos($this->template,'{'.$id.'}')===false)
				unset($this->buttons[$id]);
			else if(isset($button['click']) || isset($button['submit']))
			{
				if(!isset($button['options']['class']))
					$this->buttons[$id]['options']['class']=$id;
				if(isset($button['click']) && strpos($button['click'],'js:')!==0)
					$this->buttons[$id]['click']='js:'.$button['click'];
				if(isset($button['submit']) && isset($button['type']) && $button['type'] == self::SUBMIT_BUTTON && strpos($button['submit'],'js:')!==0)
					$this->buttons[$id]['submit']='js:'.$button['submit'];
			}
		}

		$this->registerClientScript();
	}
	
	/**
	 * Initializes the default buttons (view, create, update and delete).
	 */
	protected function initDefaultButtons()
	{
		if($this->viewButtonLabel===null)
			$this->viewButtonLabel=Yii::t('zii','View');
		if($this->updateButtonLabel===null)
			$this->updateButtonLabel=Yii::t('zii','Update');
		if($this->deleteButtonLabel===null)
			$this->deleteButtonLabel=Yii::t('zii','Delete');
		if($this->viewButtonImageUrl===null)
			$this->viewButtonImageUrl=$this->grid->baseScriptUrl.'/view.png';
		if($this->updateButtonImageUrl===null)
			$this->updateButtonImageUrl=$this->grid->baseScriptUrl.'/update.png';
		if($this->deleteButtonImageUrl===null)
			$this->deleteButtonImageUrl=$this->grid->baseScriptUrl.'/delete.png';
		if($this->deleteConfirmation===null)
			$this->deleteConfirmation=Yii::t('zii','Are you sure you want to delete this item?');
		
		if($this->createButtonLabel===null)
			$this->createButtonLabel=Yii::t('RButtonColumn','Create');

		foreach(array('view','update','delete') as $id)
		{
			$button=array(
				'label'=>$this->{$id.'ButtonLabel'},
				'url'=>$this->{$id.'ButtonUrl'},
				'imageUrl'=>$this->{$id.'ButtonImageUrl'},
				'options'=>$this->{$id.'ButtonOptions'},
			);
			if(isset($this->buttons[$id]))
				$this->buttons[$id]=array_merge($button,$this->buttons[$id]);
			else
				$this->buttons[$id]=$button;
		}
		
		$button=array(
			'label'=>$this->createButtonLabel,
			'url'=>$this->createButtonUrl,
			'imageUrl'=>$this->createButtonImageUrl,
			'options'=>$this->createButtonOptions,
		);
		if(isset($this->headerButtons['create']))
			$this->headerButtons['create']=array_merge($button,$this->headerButtons['create']);
		else
			$this->headerButtons['create']=$button;
			
		if(!isset($this->buttons['delete']['type']))
			$this->buttons['delete']['type'] = self::SUBMIT_BUTTON;
		if(!isset($this->buttons['delete']['submit']))
		{
			if(is_string($this->deleteConfirmation))
				$confirmation="if(!confirm(".CJavaScript::encode($this->deleteConfirmation).")) return false;";
			else
				$confirmation='';

			if($this->afterDelete===null)
				$this->afterDelete='function(){}';

			$this->buttons['delete']['submit']=<<<EOD
function() {
	$confirmation
	var th=this;
	var afterDelete=$this->afterDelete;
	$.fn.yiiGridView.update('{$this->grid->id}', {
		type: $(this).attr('method'),
		dataType: 'text',
		url: $(this).attr('action'),
		data: $(this).serialize(),
		success:function(data) {
			$.fn.yiiGridView.update('{$this->grid->id}');
			afterDelete(th,true,data);
		},
		error:function(XHR) {
			return afterDelete(th,false,XHR);
		}
	});
	return false;
}
EOD;
		}
	}	
	
	/**
	 * Registers the client scripts for the button column.
	 */
	protected function registerClientScript()
	{
		$js=array();
		foreach($this->buttons as $id=>$button)
		{
			$event = isset($button['submit']) ? 'submit' : (isset($button['click']) ? 'click' : null);
			if($event !== null)
			{
				$function=CJavaScript::encode($button[$event]);
				$class=preg_replace('/\s+/','.',$button['options']['class']);
				$js[]="jQuery('#{$this->grid->id} ".($event === 'submit' ? 'form' : 'a').".{$class}').live('$event',$function);";
			}
		}
		foreach($this->headerButtons as $id=>$button)
		{
			$event = isset($button['submit']) ? 'submit' : (isset($button['click']) ? 'click' : null);
			if($event !== null)
			{
				$function=CJavaScript::encode($button[$event]);
				$buttonId=preg_replace('/\s+/','.',$button['options']['id']);
				$js[]="jQuery('#{$this->grid->id} #{$buttonId}').live('$event',$function);";
			}
		}

		if($js!==array())
			Yii::app()->getClientScript()->registerScript(__CLASS__.'#'.$this->id, implode("\n",$js));
	}

	/**
	 * Renders a link button.
	 * @param string $id the ID of the button
	 * @param array $button the button configuration which may contain 'label', 'url', 'imageUrl', 'options' and 'type' elements.
	 * See {@link buttons} for more details.
	 * @param integer $row the row number (zero-based)
	 * @param mixed $data the data object associated with the row
	 */
	protected function renderButton($id,$button,$row,$data)
	{
		if (isset($button['visible']) && !$this->evaluateExpression($button['visible'],array('row'=>$row,'data'=>$data)))
  			return;
		$label=isset($button['label']) ? $button['label'] : $id;
		$url=isset($button['url']) ? $this->evaluateExpression($button['url'],array('data'=>$data,'row'=>$row)) : '#';
		$options=isset($button['options']) ? $button['options'] : array();
		if(!isset($options['title']))
			$options['title']=$label;
		if (isset($button['type']) && $button['type'] == self::SUBMIT_BUTTON) {
			$action = $url;
			if (!isset($button['url']) && isset($options['action']))
				$action = $options['action'];
			$method = 'post';
			if (isset($options['method']))
				$method = $options['method'];
			$submit = '';
			if (isset($button['imageUrl']) && is_string($button['imageUrl']))
				$submit = CHtml::imageButton($button['imageUrl'], array('class' => 'button-column-submit'));
			else
				$submit = CHtml::submitButton($label, array('class' => 'button-column-submit'));
			echo CHtml::beginForm($action, $method, $options);
			echo CHtml::tag('fieldset', array('class' => 'button-column-field'), $submit, true);
			echo CHtml::endForm();
		}
		else {
			if(isset($button['imageUrl']) && is_string($button['imageUrl']))
				echo CHtml::link(CHtml::image($button['imageUrl'],$label),$url,$options);
			else
				echo CHtml::link($label,$url,$options);
		}
	}
		
	/**
	 * Renders the filter cell (with Html options).
	 * @since 1.1.1
	 */
	public function renderFilterCell()
	{
		echo CHtml::tag('td', $this->filterHtmlOptions, false, false);
		echo $this->renderFilterCellContent();
		echo CHtml::closeTag('td');
	}
	
	/**
	 * Renders the filter cell content.
	 * This method renders any header buttons.
	 * @since 1.1.1
	 */
	protected function renderFilterCellContent() {
		$tr=array();
		ob_start();
		foreach($this->headerButtons as $id=>$button)
		{
			$this->renderButton($id,$button,-1,null);
			$tr['{'.$id.'}']=ob_get_contents();
			ob_clean();
		}
		ob_end_clean();
		echo strtr($this->headerTemplate,$tr);
	}
}